const express = require("express");
const router = express.Router();
const DecryptWithServerKey = require("./../../services/decryptionservice");
const PractitionerPatientCount = require("../../model/practitioner/practitioner-patient");
const StellarSDK = require("stellar-sdk");
const { TimeoutInfinite } = require("stellar-sdk");
const IPFS = require("ipfs-infura");
const LIMIT = 64;
const EncryptionService = require("./../../services/encryptionservice");

router.post("/", async (req, res) => {
  // const decryptedPatientPubKey = DecryptWithServerKey(req.body.patient);
  // const decryptedPractitionerPubKey = DecryptWithServerKey(req.body.practitionerpubkey);
  // const decryptedPractitionerSeed = DecryptWithServerKey(req.body.practitionerseed);
  const timestampSent = req.body.timestamp;
  const records = req.body.records;

  try {
    //get the current hash and current count for the patient practitioner relationship
    let practitionerDetails = await PractitionerPatientCount.findOne({
      practitionerpubkey: req.body.practitionerpubkey,
      patientpubkey: req.body.patient,
    });

    if (!practitionerDetails) {
      console.log("No patient-practitioner records found for the public keys");
      console.log("/practitioner/records - adding practitioner records failed");
      res.status(400).json({
        message: "No patient-practitioner records found for the public keys",
      });
    } else {
      let previousHash = practitionerDetails.currenthash;
      let currentCount = practitionerDetails.currentcounter + 1;

      //load user account
      const userSourceKeyPair = StellarSDK.Keypair.fromSecret(
        req.body.practitionerseed
      );
      const userPublicKey = userSourceKeyPair.publicKey();
      const server = new StellarSDK.Server(process.env.STELLARTESTNET);
      const account = await server.loadAccount(userPublicKey);
      const fee = await server.fetchBaseFee();

      //encrypt the weekly records using practitioner BC secret key
      const encryptedRec = EncryptionService.EncryptWithStellarSeed(
        req.body.practitionerseed
      );

      //create the IPFS instance
      const ipfs = new IPFS({
        host: process.env.IPFSHOST,
        port: process.env.IPFSPORT,
        protocol: process.env.IPFSPROTOCOL,
        projectId: process.env.IPFSPROJECTID,
        projectSecret: process.env.IPFSAPIKEY,
      });

      //add the weekly records to IPFS
      ipfs
        .add(encryptedRec)
        .then(async (response) => {
          let ipfsContentCID = response.toString();

          //build the transaction
          const transaction = new StellarSDK.TransactionBuilder(account, {
            fee,
            networkPassphrase: StellarSDK.Networks.TESTNET,
          })
            .addMemo(StellarSDK.Memo.text("Record " + currentCount.toString()))
            .addOperation(
              StellarSDK.Operation.manageData({
                name: "Timestamp",
                value: timestampSent,
              })
            )
            .addOperation(
              StellarSDK.Operation.manageData({
                name: "Previous",
                value: previousHash,
              })
            )
            .addOperation(
              StellarSDK.Operation.manageData({
                name: "CID",
                value: ipfsContentCID.toString(),
              })
            )
            .setTimeout(30)
            .build();

          //sign the transaction
          transaction.sign(userSourceKeyPair);
          //send the transaction to the blockchain
          try {
            const transactionResult = await server.submitTransaction(
              transaction
            );
            console.log(
              "Weekly record transaction added to blockchain : HASH : " +
                transactionResult.hash
            );
            console.log(
              "View weekly record transaction at : https://horizon-testnet.stellar.org/transactions/" +
                transactionResult.hash
            );
            try {
              //update the counter collection
              const updateCounter = await PractitionerPatientCount.updateOne(
                {
                  practitionerpubkey: req.body.practitionerpubkey,
                  patientpubkey: req.body.patient,
                },
                {
                  currenthash: transactionResult.hash,
                  currentcounter: currentCount,
                }
              );

              if (!updateCounter) {
                console.log("Failed to update the hash : " + updateCounter);
                console.log(
                  "/practitioner/records - adding practitioner records failed"
                );
                return res.status(500).json({
                  message: "Failed to update the hash : " + updateCounter,
                });
              } else {
                console.log(
                  "Weekly record transaction saved to database : " +
                    updateCounter
                );
                res.status(200).json({
                  message: "Weekly record transaction saved to blockchain",
                  hash: transactionResult.hash,
                });
                console.log(
                  "/practitioner/records - adding practitioner records success"
                );
              }
            } catch (errorWhenUpdatingCounter) {
              console.log(
                "Error when updating patient-practitioner count in DB : " +
                  errorWhenUpdatingCounter
              );
              res.status(500).json({
                message:
                  "Error when updating patient-practitioner count in DB : " +
                  errorWhenUpdatingCounter,
              });
              console.log(
                "/practitioner/records - adding practitioner records failed"
              );
            }
          } catch (errorWhenSendingTransactionToBlockchain) {
            console.log(
              "Error when sending weekly records to blockchain : " +
                errorWhenSendingTransactionToBlockchain
            );
            res.status(500).json({
              message:
                "Error when sending weekly records to blockchain : " +
                errorWhenSendingTransactionToBlockchain,
            });
            console.log(
              "/practitioner/records - adding practitioner records failed"
            );
          }
        })
        .catch((errorWhenAddingToIPFS) => {
          console.log(
            "Error when adding weekly record to IPFS : " + errorWhenAddingToIPFS
          );
          console.log(
            "/practitioner/records - adding practitioner records failed"
          );
          res.status(400).json({
            message:
              "Error when adding weekly record to IPFS : " +
              errorWhenAddingToIPFS,
          });
        });
    }
  } catch (errorWhenGettingCount) {
    console.log(
      "Error when getting the practitioner-patient details from the DB : " +
        errorWhenGettingCount
    );
    res.status(500).json({
      message:
        "Error when getting the practitioner-patient details from the DB : " +
        errorWhenGettingCount,
    });
    console.log("/practitioner/records - adding practitioner records failed");
  }
});

module.exports = router;
